# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import markitup.fields
import model_utils.fields
import django.utils.timezone
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Entry',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('title', models.CharField(max_length=500)),
                ('slug', models.SlugField(unique=True, editable=False)),
                ('content', markitup.fields.MarkupField(no_rendered_field=True)),
                ('is_published', models.BooleanField(default=False)),
                ('published_timestamp', models.DateTimeField(null=True, editable=False, blank=True)),
                ('_content_rendered', models.TextField(editable=False, blank=True)),
                ('author', models.ForeignKey(editable=False, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'verbose_name_plural': 'entries',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EntryImage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('image', models.ImageField(upload_to=b'andablog/images', blank=True)),
                ('entry', models.ForeignKey(to='andablog.Entry')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
    ]



from django.conf import settings
from django.conf.urls import include, url
from django.views.generic import TemplateView
from django.contrib import admin
from django.conf.urls.static import static

from andablog.sitemaps import EntrySitemap

from profiles.sitemaps import UserProfileSitemap

admin.autodiscover()

sitemaps = {
    'profiles': UserProfileSitemap,
    'blog': EntrySitemap,
}

urlpatterns = [
    url(r'^$', TemplateView.as_view(template_name='home.html')),
    url(r'^accounts/', include('allauth.urls')),  # All Auth
    url(r'^blog/', include('blog.urls')),
    url(r'^profile/', include('profiles.urls')),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^comments/', include('django_comments.urls')),

    url(r'^sitemap\.xml$', 'django.contrib.sitemaps.views.sitemap', {'sitemaps': sitemaps}),
    url(r'^markitup/', include('markitup.urls')),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT) # Static media hosting in debug mode

if settings.DEBUG:
    import debug_toolbar
    urlpatterns.append(url(r'^__debug__/', include(debug_toolbar.urls)))



"""
Since we rely on an underlying blog engine we are primarily
testing the integration points of the chosen blog engine. E.g.
 * Are the URLs from the blog engine hooked up correctly?
 * Are we including the blog data into the sitemap?
 * Are we exposing a blog feed?
"""
from django.core.urlresolvers import reverse
from django.test import TestCase

from andablog import models as blogmodels

from .feeds import LatestBlogEntries


class EntryListingTests(TestCase):
    """Posts or not, we just want to make sure we are hooking this up properly"""

    def setUp(self):
        self.url = reverse('andablog:entrylist')

    def test_rendering(self):
        """The listing should render properly"""
        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 200)


class LatestEntriesFeed(TestCase):
    """We want to make sure we hooked up the entry feed properly"""

    fixtures = ['three_users', 'three_profiles', 'three_published_entries']

    def setUp(self):
        self.feed = LatestBlogEntries()

    def test_author_details(self):
        """Test that our custom user is integrating properly"""
        an_entry = blogmodels.Entry.objects.get(slug='last-post')

        expected = (
            an_entry.published_timestamp,
            an_entry.author.profile_name,
            an_entry.author.email,
            an_entry.author.get_absolute_url()
        )

        actual = (
            self.feed.item_pubdate(an_entry),
            self.feed.item_author_name(an_entry),
            self.feed.item_author_email(an_entry),
            self.feed.item_author_link(an_entry),
        )

        self.assertEqual(expected, actual)

    def test_url(self):
        """Should be able to get the feed items by URL"""
        url = reverse('blog-entry-feed')

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)


class TestAuthorEntryListing(TestCase):
    """An author looking at the entry listing"""

    fixtures = ['three_users', 'three_profiles', 'three_published_entries']

    def setUp(self):
        an_entry = blogmodels.Entry.objects.get(slug='last-post')
        an_entry.published_timestamp = None
        an_entry.is_published = False
        an_entry.save()

        self.client.login(username='agent0014@example.com', password='secret')

        self.url = reverse('andablog:entrylist')

    def test_draft_listed(self):
        """Our author should see the draft entry."""
        response = self.client.get(self.url)

        expected_slugs = ['last-post', 'busy-busy', 'welcome']
        actual_slugs = [entry.slug for entry in response.context['entries']]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(actual_slugs, expected_slugs)
        self.assertNumQueries(1)


class TestAuthorEntryDetail(TestCase):
    """An author looking at the entry detail"""

    fixtures = ['three_users', 'three_profiles', 'three_published_entries']

    def setUp(self):
        self.an_entry = blogmodels.Entry.objects.get(slug='last-post')
        self.an_entry.published_timestamp = None
        self.an_entry.is_published = False
        self.an_entry.save()

        self.client.login(username='agent0014@example.com', password='secret')

        self.url = reverse('andablog:entrydetail', args=[self.an_entry.slug])

    def test_draft_detail(self):
        """Our author should be able to view the draft entry."""
        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(self.an_entry.slug, response.context['entry'].slug)
        self.assertNumQueries(1)


from django.conf.urls import include, url
from .feeds import LatestBlogEntries

urlpatterns = [
    url(r'^latest/entries/$', LatestBlogEntries(), name='blog-entry-feed'),
    url(r'^', include('andablog.urls', namespace='andablog')),
]


from django_comments.forms import CommentForm
from django import forms

from .models import DemoComment


class DemoCommentForm(CommentForm):

    def __init__(self, *args, **kwargs):
        super(DemoCommentForm, self).__init__(*args, **kwargs)
        self.fields['comment'].widget = forms.Textarea(attrs={'rows': 2})

    def get_comment_model(self):
        return DemoComment



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.core.urlresolvers import reverse
from django.test import TestCase

from andablog import models


class TestEntryListing(TestCase):

    def setUp(self):
        self.post_1 = models.Entry.objects.create(title=u'Welcome!', is_published=True)
        self.post_2 = models.Entry.objects.create(title=u'Busy Busy', is_published=True)
        self.post_3 = models.Entry.objects.create(title=u'Last Post', is_published=True)
        self.post_4 = models.Entry.objects.create(title=u'Back again!')

        self.url = reverse('andablog:entrylist')

    def test_anonymous_get(self):
        """Only published entries should be listed by descending published timestamp"""
        response = self.client.get(self.url)

        expected_slugs = ['last-post', 'busy-busy', 'welcome']
        actual_slugs = [entry.slug for entry in response.context['entries']]

        self.assertEqual(response.status_code, 200)
        self.assertEqual(actual_slugs, expected_slugs)
        self.assertNumQueries(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.test import TestCase
from django.utils import timezone

from andablog import models, sitemaps


class TestEntrySitemap(TestCase):

    def setUp(self):
        self.post_1 = models.Entry.objects.create(title=u'Welcome!', is_published=True)
        self.post_2 = models.Entry.objects.create(title=u'Busy Busy', is_published=True)
        self.post_3 = models.Entry.objects.create(title=u'Last Post', is_published=True)
        self.post_4 = models.Entry.objects.create(title=u'Back again!')
        self.entry_map = sitemaps.EntrySitemap()

    def test_items(self):
        """Only published entries should be listed by descending published timestamp"""
        actual_entries = self.entry_map.items()

        expected_slugs = ['last-post', 'busy-busy', 'welcome']
        actual_slugs = [entry.slug for entry in actual_entries]

        self.assertEqual(actual_slugs, expected_slugs)
        self.assertNumQueries(1)

    def test_last_modified(self):
        """Should be able to get the last update from an entry"""
        actual_update = self.entry_map.lastmod(self.post_3)

        now = timezone.now()
        self.assertGreaterEqual(now, actual_update)
        self.assertGreaterEqual(actual_update, self.post_1.modified)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.test import TestCase

from andablog import models


class TestEntryDetail(TestCase):

    def setUp(self):
        self.entry_1 = models.Entry.objects.create(title=u'Welcome!', is_published=True)

        self.url = self.entry_1.get_absolute_url()

    def test_published(self):
        """A published entry should be shown"""
        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(self.entry_1, response.context['entry'])
        self.assertNumQueries(1)

    def test_unpublished(self):
        """An unpublished entry should not be found"""
        self.entry_1.is_published = False
        self.entry_1.save()

        response = self.client.get(self.url)

        self.assertEqual(response.status_code, 404)



from django.core.urlresolvers import reverse_lazy
from andablog.feeds import LatestEntriesFeed


class LatestBlogEntries(LatestEntriesFeed):
    feed_copyright = 'Example Org'
    title = 'Latest Blog Entries'
    description = 'Updates on the latest blog entries from example.com.'
    link = reverse_lazy('andablog:entrylist')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
from django.conf import settings
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('greatest_fear', models.CharField(max_length=100, null=True, blank=True)),
                ('home', models.TextField(null=True, blank=True)),
                ('user', models.OneToOneField(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.test import TestCase
from django.utils.text import slugify
from django.utils import timezone

from andablog import models


class TestEntryModel(TestCase):
    """
    Author Note: We don't do much, if any testing, surrounding the Entry author attribute.
    As all of the integration code for them is in the templates.

    TODO: For 1.7, And-a-Blog should provide a Django system check that ensures that get_short_name and
    get_absolute_url have been implemented on the user model as that is what the templates rely on.
    """

    def setUp(self):
        self.entry = models.Entry.objects.create(title=u'First post!', content=u'The best post on the internet.')

    def test_slug_creation(self):
        """The slug field should automatically get set from the title during post creation"""
        self.assertEqual(self.entry.slug, slugify(self.entry.title))

    def test_new_duplicate(self):
        """The slug value should automatically be made unique if the slug is taken"""
        duplicate_entry = models.Entry.objects.create(title=self.entry.title, content=self.entry.content)

        self.assertNotEqual(self.entry.slug, duplicate_entry.slug)
        self.assertIn(self.entry.slug, duplicate_entry.slug)

    def test_title_rename_to_duplicate(self):
        """Upon title rename, the slug value should automatically be made unique if the slug is taken"""
        new_entry_2 = models.Entry.objects.create(title=u'Second post!', content=u'Second best post on the internet.')

        # Rename
        new_entry_2.title = self.entry.title
        new_entry_2.save()

        self.assertNotEqual(self.entry.slug, new_entry_2.slug)
        self.assertIn(self.entry.slug, new_entry_2.slug)

    def test_publishing(self):
        """Test publishing an entry"""
        self.assertFalse(self.entry.is_published)
        self.assertFalse(self.entry.published_timestamp)

        moment_ago = timezone.now()

        self.entry.is_published = True
        self.entry.save()

        self.assertGreaterEqual(self.entry.published_timestamp, moment_ago)

    def test_unpublishing(self):
        """Should be able to pull an entry after it has been published"""
        self.entry.is_published = True
        self.entry.save()

        self.entry.is_published = False
        self.entry.save()

        self.assertFalse(self.entry.is_published)
        self.assertIsNone(self.entry.published_timestamp)



from django.contrib.sitemaps import Sitemap

from . import models


class UserProfileSitemap(Sitemap):

    changefreq = "weekly"
    priority = 0.5
    protocol = 'http'

    def items(self):
        return models.UserProfile.objects.filter(user__is_active=True).order_by('-user__date_joined')

    def lastmod(self, obj):
        return obj.modified



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.test import TestCase
from django.utils.text import slugify

from . import models


class TestCustomUserModel(TestCase):

    def setUp(self):
        self.user = models.User.objects.create(
            name=u'Clark Jerome Kent',
            profile_name=u'Superman',
            email=u'manofsteel1938@example.com',
        )

    def test_slug_creation(self):
        """The slug field should automatically get set from the profile name during user creation"""
        self.assertEqual(self.user.slug, slugify(self.user.profile_name))

    def test_short_name(self):
        """The short name function should be the profile name"""
        self.assertEqual(self.user.profile_name, self.user.get_short_name())



from django.contrib.sitemaps import Sitemap

from .models import Entry


class EntrySitemap(Sitemap):
    """
    Sitemap for entries.
    """
    priority = 0.5
    changefreq = 'weekly'

    def items(self):
        """
        Return published entries.
        """
        return Entry.objects.filter(is_published=True).order_by('-published_timestamp')

    def lastmod(self, obj):
        """
        Return last modification of an entry.
        """
        return obj.modified



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

requirements = [
    'six',
    'Django>=1.7,<1.9',
    'django-model-utils>=2.2',
    'django-markitup>=2.2.2',
    'django-taggit>=0.14.0',
    'pillow>=2.6.1',
]

test_requirements = [
    # None, these go into the test_requirements file
]

setup(
    name='django-andablog',
    version='1.4.2',
    description='A blog app that is only intended to be embedded within an existing Django site.',
    long_description=readme + '\n\n' + history,
    author='Ivan Ven Osdel',
    author_email='ivan@wimpyanalytics.com',
    url='https://github.com/WimpyAnalytics/django-andablog',
    download_url='https://github.com/wimpyanalytics/django-andablog/tarball/1.4.2',
    packages=[
        'andablog',
    ],
    package_dir={'andablog':
                 'andablog'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords=['django-andablog', 'blog', 'django', 'app', 'reusable app', 'andablog'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Framework :: Django',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
    ],
    test_suite='tests',
    tests_require=test_requirements,
)



from django import template
from django.utils.safestring import mark_safe
import six

register = template.Library()


@register.filter(name='authordisplay')
def author_display(author, *args):
    """Returns either the linked or not-linked profile name."""

    # Call get_absolute_url or a function returning none if not defined
    url = getattr(author, 'get_absolute_url', lambda: None)()
    # get_short_name or unicode representation
    short_name = getattr(author, 'get_short_name', lambda: six.text_type(author))()
    if url:
        return mark_safe('<a href="{}">{}</a>'.format(url, short_name))
    else:
        return short_name



from django.contrib import admin

from .models import DemoComment


class DemoCommentAdmin(admin.ModelAdmin):
    list_display = (
        'user',
        'submit_date',
    )
    search_fields = (
        'user__profile_name',
        'user__email',
        'comment',
    )

admin.site.register(DemoComment, DemoCommentAdmin)



from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^$', views.profile_redirect, name='profile-detail-redirect'),
    url(r'^(?P<slug>[A-Za-z0-9-_]+)/$', views.UserProfileDetail.as_view(), name='profile-detail'),
]



import django_comments.models as dmodels


class RelatedCommentManager(dmodels.CommentManager):

    def get_queryset(self):
        return super(RelatedCommentManager, self).get_queryset().select_related(
            'user',
            'user__profile'
        )


class DemoComment(dmodels.Comment):

    class Meta:
        proxy = True
        app_label = 'democomments'

    objects = RelatedCommentManager()
